//=============================================================================
// FoGsesipod - Headspace Expanded BASE
// HeadspaceExpanded-BASE.js
//=============================================================================

//=============================================================================
/*:
 * @plugindesc Base Functions and Modifications for Headspace Expanded
 * @author FoGsesipod
 * @help
 *-----------------------------------------------------------------------------
 * ★ Headspace Expanded ★
 *-----------------------------------------------------------------------------
 * ★ Event Notetags ★
 *-----------------------------------------------------------------------------
 * <canPass Override> - Removes Event Hitbox Resizes Immobility
 * 
 *-----------------------------------------------------------------------------
 * ★ Regions ★
 *-----------------------------------------------------------------------------
 * Region 21 - Blocks Events Including Those With Through
 * Region 25 - Same As Above But Also Acts As A Ladder For The Party
 * 
 * -----------------------------------------------------------------------------
 * ★ Plugin Commands ★
 * -----------------------------------------------------------------------------
 * ShowChoices -1 - Disallows Cancelling
 * 
 * -----------------------------------------------------------------------------
 * ★ Yaml Messages ★
 * -----------------------------------------------------------------------------
 * \Eval{code} - Allows Evaling Javascript Code Once A Message Is Displayed
 * 
*/
//=============================================================================

// Event Hitbox Resize canPass Override
old_EventHitboxResizeCanPass = Game_CharacterBase.prototype.canPass;
Game_CharacterBase.prototype.canPass = function(x, y, d) {
  if (this._canPassOverride) {return Yanfly.EvHbRs.Game_CharacterBase_canPass.call(this, x, y, d)}
  return old_EventHitboxResizeCanPass.call(this, x, y, d);
}
  
old_EventHitboxResizeSetupPage = Game_Event.prototype.setupPage;
Game_Event.prototype.setupPage = function() {
  old_EventHitboxResizeSetupPage.call(this);

  this._addedHitboxUp = 0;
  this._addedHitboxLeft = 0;
  this._addedHitboxRight = 0;
  this._addedHitboxDown = 0;
  this._canPassOverride = 0;

  this.setupEventHitboxResizeNotetags();
  this.setupEventHitboxResizeCommentTags();
};

old_SetupEventHitboxResizeNotetags = Game_Event.prototype.setupEventHitboxResizeNotetags;
Game_Event.prototype.setupEventHitboxResizeNotetags = function() {
  old_SetupEventHitboxResizeNotetags.call(this);
  if (this.event().note.match(/<canPass Override>/i)) {
    this._canPassOverride = 1
  }
};

// Region 21 and 25 Blocking Through Events - Thank You Draught
{
  const old_isEventRegionForbid = Game_CharacterBase.prototype.isEventRegionForbid
  Game_CharacterBase.prototype.isEventRegionForbid = function(x,y,d) {
    if (this instanceof Game_Follower) {return false}
    if (this.isPlayer()) {return false}
    var regionId = this.getRegionId(x, y, d);
    if (regionId === 21 || regionId === 25) {return true}
    return old_isEventRegionForbid.call(this, x, y, d)
  }
}

{
  let old_canPass = Game_CharacterBase.prototype.canPass
  Game_CharacterBase.prototype.canPass = function(x, y, d) {
    if (this.isEventRegionForbid(x,y,d)) {
      return false
    }
    return old_canPass.call(this, x,y,d)
  }
}

// Region 25 Is Also A Ladder
{
  const old_isLadder = Game_Map.prototype.isLadder
  Game_Map.prototype.isLadder = function(x, y) {
    if (this.regionId(x, y) === 25) {
      return this.isValid(x, y) && true;
    }
  	return old_isLadder.call(this, x, y);
  };

  const old_isClimbing = Game_CharacterBase.prototype.isClimbing
  Game_CharacterBase.prototype.isClimbing = function() {
    if (this.regionId() === 25) {
      return true
    }
    return old_isClimbing.call(this)
  }
}

// Fix ShowChoices Plugin Command So We Can Use Disallow Cancel Type - Thank You Draught
{
  let old = Game_Interpreter.prototype.setupLanguageChoices
  Game_Interpreter.prototype.setupLanguageChoices = function(cancel) {
    return old.call(this, parseInt(cancel))
  }
}

// Save Load Plus Background Fix For New Save Locations
// Currently unused artifact of previous generation Mirror World
{
  if ($modLoader.knownMods.has("saveloadplus") || $modLoader.knownMods.has("SaveLoadPlus")) {
    const old_getBackground = getBackground
    window['getBackground'] = function(location) { 
      switch(location){
      case "DNUORGYALP TSEROF":
        return [0,2];
      }
    return  old_getBackground.call(this, location)
  }}
}

// Allows /Eval(code) in yaml messages - Thank you Draught and Rph
// Unsure where to use this or if i will, Its very buggy for complex Eval tasks
{
  // let old = Window_Base.prototype.convertEscapeCharacters
  // Window_Base.prototype.convertEscapeCharacters = function(text) {
  //     text = old.call(this, text)
  //     return text.replace(/\x1bEval\{.+\}(?<!\\\})/, function() {
  //         let code = arguments[0]
  //         if (code) {
  //           let c = code.match(/^\x1bEval{(.+)}$/)[1];
  //           console.log("cleancode", c)
  //             eval(c)
  //         }
  //         return ''
  //     })
  // };

  let old = Window_Base.prototype.convertEscapeCharacters
  Window_Base.prototype.convertEscapeCharacters = function(text) {
      text = text.replace(/\\Eval\{.+\}(?<!\\\})/, function() {
          let code = arguments[0]
          if (code) {
            console.log("code", code)
            let c = code.match(/\{.+\}/)[0];
            console.log("cleancode", c)
              eval(c)
          }
          return ''
      })
      return old.call(this, text)
  };
}